package com.qs.mp.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.common.enums.CouponTypeEnum;
import com.qs.mp.common.enums.UserCouponStatusEnum;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.user.domain.UserCoupon;
import com.qs.mp.user.domain.vo.UserCouponVO;
import com.qs.mp.user.mapper.UserCouponMapper;
import com.qs.mp.user.service.IUserCouponService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import java.util.List;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 用户的优惠券 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-07
 */
@Service
public class UserCouponServiceImpl extends ServiceImpl<UserCouponMapper, UserCoupon> implements IUserCouponService {

  @Override
  public List<UserCouponVO> listTicketOrderCoupon(Long userId) {
    List<UserCouponVO> userCouponVOList = getBaseMapper().listUserCouponVO(new QueryWrapper<UserCoupon>()
        .eq("t1.user_id", userId).eq("t2.type", CouponTypeEnum.TICKET_ORDER)
        .eq("t1.status", UserCouponStatusEnum.UNUSED)
        .le("t1.valid_start", DateUtils.getToday()).ge("t1.valid_end", DateUtils.getToday())
        .orderByDesc("t2.discount"));
    return userCouponVOList;
  }

  @Override
  public List<UserCouponVO> listTicketOrderCouponByIds(List<String> userCouponIds) {
    List<UserCouponVO> userCouponVOList = getBaseMapper().listUserCouponVO(new QueryWrapper<UserCoupon>()
        .in("t1.id", userCouponIds));
    return userCouponVOList;
  }

  @Override
  public List<UserCouponVO> listCoupon(Long userId, UserCouponStatusEnum status) {
    QueryWrapper<UserCoupon> queryWrapper = new QueryWrapper<UserCoupon>()
        .eq("t1.user_id", userId)
        .orderByAsc("t1.valid_end");
    if (status == UserCouponStatusEnum.UNUSED) {
      queryWrapper.eq("t1.status", UserCouponStatusEnum.UNUSED)
          .le("t1.valid_start", DateUtils.getToday()).ge("t1.valid_end", DateUtils.getToday());
    } else if (status == UserCouponStatusEnum.OVERDUE) {
      queryWrapper.eq("t1.status", UserCouponStatusEnum.UNUSED)
          .lt("t1.valid_end", DateUtils.getToday());
    } else if (status == UserCouponStatusEnum.USED) {
      queryWrapper.eq("t1.status", UserCouponStatusEnum.USED);
    }
    List<UserCouponVO> userCouponVOList = getBaseMapper().listUserCouponVO(queryWrapper);
    return userCouponVOList;
  }

  @Override
  public List<UserCouponVO> listUserCouponVO(Wrapper<UserCoupon> queryWrapper) {
    return getBaseMapper().listUserCouponVO(queryWrapper);
  }
}
